@@ -14,6 +14,7 @@ from uuid import uuid4
 
 import IPython
 import botocore
+from botocore.config import Config as BotocoreConfig
 import filelock
 import sparkmagic.utils.configuration as conf
 from IPython import get_ipython
@@ -33,6 +34,7 @@ from sagemaker_studio_sparkmagic_lib.emr import EMRCluster
 from sagemaker_studio_sparkmagic_lib.kerberos import write_krb_conf
 
 from ..resource.emr.auth import ClusterSessionCredentialsProvider
+from ..resource.emr.auth import ClusterAuthUtils
 from ..resource.emr.emr_client import get_emr_client
 from ..utils.arn_util import RoleArnValidator
 from ..utils.boto_client_utils import get_boto3_session
@@ -62,6 +64,8 @@ from ..utils.service_metrics import (
     records_service_metrics,
 )
 
+from ..utils.string_utils import is_blank
+
 # This logger emits logs to sys.stdout, which is visible in customer's notebook in Studio
 sys_out_logger = logging.getLogger(__name__)
 sys_out_logger.setLevel(logging.INFO)
@@ -111,12 +115,11 @@ def _magic_kernel_connect_to_emr(
 def _get_endpoint_magic_line(
     emr_client, args, emr_cluster, username, password, kernel_name
 ):
-    livy_host = emr_cluster.primary_node_private_dns_name()
     livy_port = _get_livy_port(
         args=args, emr_client=emr_client, emr_cluster=emr_cluster
     )
 
-    livy_endpoint = _prepare_livy_endpoint(livy_host, int(livy_port))
+    livy_endpoint = _prepare_livy_endpoint(emr_cluster, int(livy_port))
     if kernel_name in MAGIC_KERNELS:
         return _get_magic_kernel_endpoint_magic_line(
             args=args,
@@ -133,17 +136,37 @@ def _get_endpoint_magic_line(
     )
 
 
-def _prepare_livy_endpoint(livy_host, livy_port):
+def _prepare_livy_endpoint(emr_cluster, livy_port):
     # fast fail by check livy port availability. If fail, do not proceed to compose the endpoint.
-    if not check_host_and_port(livy_host, livy_port):
-        raise RuntimeError(f"Cannot connect to livy service: {livy_host}:{livy_port}")
+    private_livy_host_name = emr_cluster.primary_node_private_dns_name()
+    if check_host_and_port(private_livy_host_name, livy_port):
+        livy_host_name = private_livy_host_name
+    else:
+        # Check public dns connectivity only if connection to private dns fails
+        public_livy_host_name = emr_cluster.primary_node_public_dns_name()
+        if is_blank(public_livy_host_name):
+            raise RuntimeError(
+                f"Cannot connect to livy service at {private_livy_host_name}:{livy_port}"
+            )
+
+        if check_host_and_port(public_livy_host_name, livy_port):
+            # Fail if public dns is accessible but not configured to use SSL
+            if not is_ssl_enabled(public_livy_host_name, livy_port):
+                raise RuntimeError(
+                    f"Livy is available at {public_livy_host_name}:{livy_port}, but not configured to use HTTPS. Please setup livy with HTTPS: https://docs.aws.amazon.com/emr/latest/ReleaseGuide/enabling-https.html"
+                )
+            livy_host_name = public_livy_host_name
+        else:
+            raise RuntimeError(
+                f"Cannot connect to livy service at {private_livy_host_name}:{livy_port} or {public_livy_host_name}:{livy_port}"
+            )
 
     protocol = "http"
-    if is_ssl_enabled(livy_host, livy_port):
+    if is_ssl_enabled(livy_host_name, livy_port):
         protocol = "https"
         _enable_ignore_ssl_errors()
 
-    livy_endpoint = "{0}://{1}:{2}".format(protocol, livy_host, livy_port)
+    livy_endpoint = "{0}://{1}:{2}".format(protocol, livy_host_name, livy_port)
     return livy_endpoint
 
 
@@ -261,6 +284,7 @@ class IpythonWidgetBasedAuthProvider:
 
 
 def _connect_to_cluster(emr_client, args, emr_cluster, kernel_name, username, password):
+    # For LDAP/HTTP authentication
     if args.auth_type == AUTH_TYPE_BASIC_ACCESS:
         _initiate_connect_based_on_kernel(
             emr_client=emr_client,
@@ -270,6 +294,7 @@ def _connect_to_cluster(emr_client, args, emr_cluster, kernel_name, username, pa
             password=password,
             kernel_name=kernel_name,
         )
+    # For Kerberos authentication
     else:
         # At this point lock should be available to acquire if another process has not already taken it
         # Go ahead and try to use the "with" context directly
@@ -501,29 +526,30 @@ def _validate_emr_args(args, usage, kernel_name):
             )
 
 
-def _initiate_emr_connect(args, boto_session, kernel_name):
-    emr_client = get_emr_client(boto_session)
+def _initiate_emr_connect(args, cluster, emr_client, boto_session, kernel_name):
 
-    """
-    Even if assumable role arn is None, we can pass it to the EMRCluster constructor and will be handled by
-    get_boto3_session method call gracefully. So no need to do a explicit None check here.
-    """
-    cluster = EMRCluster(
-        cluster_id=args.cluster_id, role_arn=args.assumable_role_arn, emr=emr_client
-    )
     _validate_cluster_auth_with_auth_type_provided(
         auth_type=args.auth_type, emr_cluster=cluster
     )
 
+    # Please update get_auth_type_for_logging() in ../emr/auth.py when updating below branches
+
     # If emr execution role is passed, the EMR IAM passthrough should be used and the token API is invoked
     # to get user and password which are later used for basic auth.
+    #
+    # For RBAC authentication
     if args.auth_type == AUTH_TYPE_BASIC_ACCESS and args.emr_execution_role_arn:
-        # Before EMR IAM Passthrough is GA, use emr-iam-passthrough-beta as service.
-        beta_emr_client = boto_session.client("emr-iam-passthrough-beta")
+        # Create a boto session config to append "sagemaker-analytics" to user-agent
+        emr_boto_config = BotocoreConfig(user_agent_extra="sagemaker-analytics")
+
+        # Before EMR IAM Passthrough is GA, use emr-iam-passthrough-gcsc as service.
+        emr_gcsc_client = boto_session.client(
+            "emr-iam-passthrough-gcsc", config=emr_boto_config
+        )
 
         credentials = (
             ClusterSessionCredentialsProvider().get_cluster_session_credentials(
-                beta_emr_client, args.cluster_id, args.emr_execution_role_arn
+                emr_gcsc_client, args.cluster_id, args.emr_execution_role_arn
             )
         )
         # wait for 3s for EMR token propagation
@@ -537,6 +563,7 @@ def _initiate_emr_connect(args, boto_session, kernel_name):
             password=credentials.password,
             kernel_name=kernel_name,
         )
+    # For HTTP-Basic/LDAP/Kerberos authentication
     elif cluster.is_krb_cluster or args.auth_type == AUTH_TYPE_BASIC_ACCESS:
         auth_provider = IpythonWidgetBasedAuthProvider()
         auth_provider.get_credentials_and_connect_to_cluster(
@@ -545,6 +572,7 @@ def _initiate_emr_connect(args, boto_session, kernel_name):
             emr_cluster=cluster,
             kernel_name=kernel_name,
         )
+    # For no authentication
     else:
         _initiate_connect_based_on_kernel(
             emr_client=emr_client,
@@ -602,7 +630,7 @@ def _set_logging_config() -> None:
 
 def _validate_cluster_auth_with_auth_type_provided(auth_type, emr_cluster):
     is_cluster_kerberos_authenticated = emr_cluster.is_krb_cluster
-    is_cluster_ldap_authenticated = _is_cluster_ldap(emr_cluster)
+    is_cluster_ldap_authenticated = ClusterAuthUtils.is_cluster_ldap(emr_cluster)
     is_cluster_no_auth = (
         not is_cluster_ldap_authenticated and not is_cluster_kerberos_authenticated
     )
@@ -625,26 +653,6 @@ def _check_required_args(args, usage):
         )
 
 
-def _is_cluster_ldap(cluster):
-    emr_configuration_list = cluster.__dict__.get("_cluster").get("Configurations")
-    if emr_configuration_list is None or not emr_configuration_list:
-        return False
-
-    # For cluster configuration
-    for configuration in emr_configuration_list:
-        if (
-            "Classification" in configuration
-            and configuration["Classification"] == "livy-conf"
-            and "Properties" in configuration
-        ):
-            livy_properties = configuration["Properties"]
-            if "livy.server.auth.type" in livy_properties:
-                livy_server_auth_type = livy_properties["livy.server.auth.type"]
-                if livy_server_auth_type == "ldap":
-                    return True
-    return False
-
-
 @magics_class
 class SagemakerAnalytics(Magics):
     @line_magic
@@ -729,11 +737,27 @@ class SagemakerAnalytics(Magics):
                         )
                     )
                     raise e
+
+                _validate_emr_args(args=args, usage=usage, kernel_name=kernel_name)
+                # Only create boto session if absolutely needed
+                boto_session = get_boto3_session(args.assumable_role_arn)
+                emr_client = get_emr_client(boto_session)
+
+                # Even if assumable role arn is None, we can pass it to the EMRCluster constructor and will be handled
+                # by get_boto3_session method call gracefully. So no need to do a explicit None check here.
+                cluster = EMRCluster(
+                    cluster_id=args.cluster_id,
+                    role_arn=args.assumable_role_arn,
+                    emr=emr_client,
+                )
+
                 _connect_to_emr(
                     service=service,
                     operation=operation,
                     args=args,
-                    usage=usage,
+                    cluster=cluster,
+                    emr_client=emr_client,
+                    boto_session=boto_session,
                     kernel_name=kernel_name,
                     service_logger=sagemaker_analytics_service_logger,
                 )
@@ -744,13 +768,23 @@ class SagemakerAnalytics(Magics):
 
 
 @records_service_metrics
-def _connect_to_emr(service, operation, args, usage, kernel_name, service_logger):
+def _connect_to_emr(
+    service,
+    operation,
+    args,
+    cluster,
+    emr_client,
+    boto_session,
+    kernel_name,
+    service_logger,
+):
     try:
-        _validate_emr_args(args=args, usage=usage, kernel_name=kernel_name)
-        # Only create boto session if absolutely needed
-        boto_session = get_boto3_session(args.assumable_role_arn)
         _initiate_emr_connect(
-            args=args, boto_session=boto_session, kernel_name=kernel_name
+            args=args,
+            cluster=cluster,
+            emr_client=emr_client,
+            boto_session=boto_session,
+            kernel_name=kernel_name,
         )
     except Exception as e:
         _echo_response_to_iopub_web_socket(
