@@ -4,6 +4,8 @@
 import socket
 from ssl import get_server_certificate, SSLError
 
+from sagemaker_studio_analytics_extension.utils.string_utils import *
+
 
 def check_host_and_port(host, port):
     """
@@ -12,6 +14,11 @@ def check_host_and_port(host, port):
     :param port: port to check
     :return: True or False indicate if port is alive
     """
+
+    if is_blank(host):
+        print(f"[Error] Host must not be empty.")
+        return False
+
     with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
         # timeout as 5 seconds
         sock.settimeout(5)
